﻿//WindowsBase, WindowsFormsIntegration, PresentationCore, PresentationFramework を参照に追加する
using System.IO;
using System.Windows.Forms.Integration;	//ElementHost
using System.Windows.Controls;	//Image
using System.Windows.Media;	//Stretch
using System.Windows.Media.Imaging;	//BitmapImage,BitmapCacheOption

namespace FormAndWpfTest
{
	enum EQuality
	{
		High = BitmapScalingMode.Fant,
		Normal = BitmapScalingMode.Linear,
		Low = BitmapScalingMode.NearestNeighbor
	}

	enum EFlip
	{
		X, Y
	}

	enum ERotate : int
	{
		R45 = 45,
		R90 = 90,
		L45 = -45,
		L90 = -90
	}

	//WPFのImageコントロールをホストするElementHost
	class WpfImageHost : ElementHost
	{
		public Image Image { get { return (this.Child as Image); } }
		public BitmapImage BitmapImage { get { return (Image.Source as BitmapImage); } }

		//コンストラクタ
		public WpfImageHost() : base()
		{
			//Imageを作成し、ElementHostにセットする
			Image img = new Image();
			img.Stretch = Stretch.Uniform;	//縦横比を保ってImageがElementHostにフィットする
			this.Child = img;

			TransformReset();
		}

		//BitmapImageを作成し、Imageにセットする
		public bool SetBitmapImage(string filePath)
		{
			bool isOK = true;
			try
			{
				using (FileStream fs = File.OpenRead(filePath))
				{
					isOK = SetBitmapImage(fs);
				}
			}
			catch { isOK = false; }
			return isOK;
		}
		
		//BitmapImageを作成し、Imageにセットする
		//・引数はFileStream,MemoryStreamを想定。
		public bool SetBitmapImage(Stream stream)
		{
			BitmapImage bmp = new BitmapImage();
			try
			{
				bmp.BeginInit();
				bmp.StreamSource = stream;
				bmp.CacheOption = BitmapCacheOption.OnLoad;
				bmp.CreateOptions = BitmapCreateOptions.IgnoreColorProfile;	//…(*)
				bmp.EndInit();
			}
			catch { bmp = null; return false; }
			if (bmp.CanFreeze) bmp.Freeze();

			Image.Source = bmp;
			return true;

			/*	(*)…
				ColorProfileが仕込まれた特定のExifファイルで例外発生。
				→System.Runtime.InteropServices.COMException はハンドルされませんでした。
				IgnoreColorProfileオプションを付けることでエラーは発生しなくなった。
			*/
		}

		//画質を設定する
		//http://msdn.microsoft.com/ja-jp/library/system.windows.media.bitmapscalingmode(v=vs.90).aspx
		public void SetImageQuality(EQuality quality)
		{
			RenderOptions.SetBitmapScalingMode(Image, (BitmapScalingMode)quality);
		}

		//変形
		//http://msdn.microsoft.com/ja-jp/library/system.windows.uielement.rendertransform(v=vs.90).aspx
		//http://msdn.microsoft.com/ja-jp/library/system.windows.media.transformgroup(v=vs.90).aspx
		private TransformGroup transformGroup;

		private void DoTransform(Transform t)
		{
			transformGroup.Children.Remove(trMoveCenter);
			transformGroup.Children.Add(t);
			transformGroup.Children.Add(trMoveCenter);
			Image.RenderTransform = transformGroup;
		}

		//変形：回転
		//http://msdn.microsoft.com/ja-jp/library/system.windows.media.rotatetransform(v=vs.90).aspx
		public void TransformRotate(ERotate rotateType) { TransformRotate((double)rotateType); }
		public void TransformRotate(double degree)
		{
			while (degree < 0) degree += 360;
			RotateTransform r = new RotateTransform(degree);
			DoTransform(r);
		}

		//変形：反転
		//http://msdn.microsoft.com/ja-jp/library/system.windows.media.scaletransform(v=vs.90).aspx
		private readonly ScaleTransform FlipX = new ScaleTransform(-1, 1);
		private readonly ScaleTransform FlipY = new ScaleTransform(1, -1);
		public void TransformFlip(EFlip flipType)
		{
			ScaleTransform f = flipType == EFlip.X ? FlipX : FlipY;
			DoTransform(f);
		}

		//変形：拡大縮小（scale=1.0は拡大率100％を表す）
		private double currentScale;
		private ScaleTransform trZoom;
		public void TransformZoom(double scale)
		{
			currentScale = scale;
			transformGroup.Children.Remove(trZoom);
			trZoom = new ScaleTransform(scale, scale);
			DoTransform(trZoom);
		}

		//変形：移動
		//http://msdn.microsoft.com/ja-jp/library/system.windows.media.translatetransform(v=vs.90).aspx
		private double totalX, totalY;
		private TranslateTransform trMoveCenter;	//変形の中心点の総移動量
		public void TransformOffset(double offsetX, double offsetY)
		{
			totalX += offsetX;
			totalY += offsetY;
			transformGroup.Children.Remove(trMoveCenter);
			trMoveCenter = new TranslateTransform(totalX, totalY);
			transformGroup.Children.Add(trMoveCenter);
			Image.RenderTransform = transformGroup;
		}

		//変形リセット
		public void TransformReset()
		{
			currentScale = 1.0;
			trZoom = new ScaleTransform(currentScale, currentScale);

			totalX = totalY = 0.0;
			trMoveCenter = new TranslateTransform(totalX, totalY);

			transformGroup = new TransformGroup();
			Image.RenderTransform = Transform.Identity;	//単位行列
			Image.RenderTransformOrigin = new System.Windows.Point(0.5, 0.5);
			/*
			参考	http://msdn.microsoft.com/ja-jp/library/system.windows.uielement.rendertransformorigin(v=vs.90).aspx
			RenderTransformOriginは(0.0～1.0)で原点を指定する。例えば(0.5, 0.5)はオブジェクトの中心を表す。
			1.0を超えると、オブジェクトの右・下方面の外の領域に原点を設定したことになる。
			0.0を下回ると、オブジェクトの左・上方面の外の領域に原点を設定したことになる。
			*/
		}

	}	//class
}	//namespace
